@@ -11,18 +11,25 @@
 # -sort.chr Whether chromosome names should be sorted (first sort by numbers then by letters) when reading cytoband data.
 #           Pass to `read.cytoband`.
 # -chromosome.index Index of chromosomes. The index is used only for subsetting, not for re-ordering.
-# -major.by     Increment of major ticks.
+# -major.by     Increment of major ticks. Pass to `circos.genomicInitialize`.
 # -plotType     Which tracks should be drawn. ``rect`` for ideogram rectangle, ``axis`` for genomic axis and ``labels`` for chromosome names.
 #               If it is set to ``NULL``, the function just initialize the plot but draw nothing.
+# -track.height Height of the track which contains "axis" and "labels".
+# -ideogram.height Height of the ideogram track
 # -...    Pass to `circos.initialize`
 #
 # == details
-# The function will initialize the circos plot in which each sector corresponds a chromosome. You can control the order of 
-# chromosomes by set a special format of ``cytoband`` (please refer to `read.cytoband` to find out how to set a proper ``cytoband`` value).
+# The function will initialize the circos plot in which each sector corresponds to a chromosome. You can control the order of 
+# chromosomes by set a special format of ``cytoband`` (please refer to `read.cytoband` to find out how to control a proper ``cytoband``).
+#
+# The function finally pass data to `circos.genomicInitialize` to initialize the circos plot.
+#
+# The style of ideogram is almost fixed, but you can customize it with your self-sefined code. Refer to vignette for demonstration.
 circos.initializeWithIdeogram = function(cytoband = paste(system.file(package = "circlize"),
 	"/extdata/cytoBand.txt", sep=""), species = NULL, sort.chr = TRUE,
 	chromosome.index = NULL, major.by = NULL,
-	plotType = c("ideogram", "axis", "labels"), ...) {
+	plotType = c("ideogram", "axis", "labels"), 
+	track.height = 0.05, ideogram.height = 0.05, ...) {
 	
 	cytoband = read.cytoband(cytoband, species = species, sort.chr = sort.chr)
 	df = cytoband$df
@@ -31,7 +38,7 @@ circos.initializeWithIdeogram = function(cytoband = paste(system.file(package =
 	if(! is.null(chromosome.index)) {
 		chromosome = chromosome[chromosome %in% chromosome.index]
 		if(length(chromosome) == 0) {
-			stop("Cannot find any chromosome. It is probably related to whether your chromosome names have 'chr' prefix.\nYou can run `circos.info()` to find out which chromosome names are being used.\n")
+			stop("Cannot find any chromosome. It is probably related with your chromosome names with or without 'chr' prefix.\nYou can run `circos.info()` to find out which type of chromosome names are used.\n")
 		}
 	}
 	
@@ -45,13 +52,13 @@ circos.initializeWithIdeogram = function(cytoband = paste(system.file(package =
 	# we do not need 'chr' prefix if it exits, it holds too much space.
 	sn = gsub("chr", "", sn)
 	
-	circos.genomicInitialize(df, sector.names = sn, major.by = major.by, plotType = plotType, ...)
+	circos.genomicInitialize(df, sector.names = sn, major.by = major.by, plotType = plotType, track.height = track.height, ...)
 
 	if(any(plotType %in% "ideogram")) {
 		o.cell.padding = circos.par("cell.padding")
 		circos.par(cell.padding = c(0, 0, 0, 0))
 	
-		circos.genomicTrackPlotRegion(df, ylim = c(0, 1), bg.border = NA, track.height = 0.05,
+		circos.genomicTrackPlotRegion(df, ylim = c(0, 1), bg.border = NA, track.height = ideogram.height,
 			panel.fun = function(region, value, ...) {
 				col = cytoband.col(value[[2]])
 				circos.genomicRect(region, value, ybottom = 0, ytop = 1, col = col, border = NA, ...)
@@ -68,14 +75,16 @@ circos.initializeWithIdeogram = function(cytoband = paste(system.file(package =
 #
 # == param
 # -data         A data frame containing genomic data.
-# -sector.names Names for each sectors which will be drawn along each sector
-# -major.by     Increment of major ticks. It is calculated automatically if the value is not set.
-# -plotType     Which part should be drawn. ``axis`` for genomic axis and ``labels`` for chromosome names
-# -tickLabelsStartFromZero whether axis tick labels start from 0? This will not affect x-values in cells.
+# -sector.names Labels for each sectors which will be drawn along each sector. It will not modify values of sector index.
+# -major.by     Increment of major ticks. It is calculated automatically if the value is not set (about every 10 degrees there is a major tick).
+# -plotType     If it is not ``NULL``, there will create a new track containing axis and names for sectors.
+#               This argument controls which part should be drawn, ``axis`` for genomic axis and ``labels`` for chromosome names
+# -tickLabelsStartFromZero Whether axis tick labels start from 0? This will only affect the axis labels while not affect x-values in cells.
+# -track.height If ``PlotType`` is not ``NULL``, height of the annotation track.
 # -...          Pass to `circos.initialize`
 #
 # == details
-# The function will initialize circos plot from genomic data provided. If ``plotType`` is set with value in ``axis`` or ``labels``, there will
+# The function will initialize circos plot from genomic data. If ``plotType`` is set with value in ``axis`` or ``labels``, there will
 # create a new track.
 #
 # The order of sectors related to data structure of ``data``. If the first column in ``data`` is a factor, the order of sectors
@@ -83,7 +92,8 @@ circos.initializeWithIdeogram = function(cytoband = paste(system.file(package =
 #
 # For more details on initializing genomic plot, please refer to the vignettes.
 circos.genomicInitialize = function(data, sector.names = NULL, major.by = NULL,
-	plotType = c("axis", "labels"), tickLabelsStartFromZero = TRUE, ...) {
+	plotType = c("axis", "labels"), tickLabelsStartFromZero = TRUE, 
+	track.height = 0.05, ...) {
 	
 	if(is.factor(data[[1]])) {
 		fa = levels(data[[1]])
@@ -112,7 +122,7 @@ circos.genomicInitialize = function(data, sector.names = NULL, major.by = NULL,
 	
 	# axis and chromosome names
 	if(any(plotType %in% c("axis", "labels"))) {
-		circos.genomicTrackPlotRegion(data, ylim = c(0, 1), bg.border = NA, track.height = 0.05,
+		circos.genomicTrackPlotRegion(data, ylim = c(0, 1), bg.border = NA, track.height = track.height,
 			panel.fun = function(region, value, ...) {
 				sector.index = get.cell.meta.data("sector.index")
 				xlim = get.cell.meta.data("xlim")
@@ -123,10 +133,10 @@ circos.genomicInitialize = function(data, sector.names = NULL, major.by = NULL,
 					offset = 0
 				}
 				if(is.null(major.by)) {
-					xplot = get.cell.meta.data("xplot")
-					n = round(abs(xplot[2] - xplot[1])/10)
-					major.at = pretty(xlim - offset, n) + offset
-					major.by = major.at[2] - major.at[1]
+					xlim = get.cell.meta.data("xlim")
+					major.by = .default.major.by()
+					major.at = seq(floor((xlim[1]-offset)/major.by)*major.by+offset, xlim[2], by = major.by)
+					major.at = c(major.at, major.at[length(major.at)] + major.by)
 				} else {
 					major.at = seq(xlim[1], 10^nchar(round(max(x2 - x1 + 1))), by = major.by)
 				}
@@ -143,7 +153,7 @@ circos.genomicInitialize = function(data, sector.names = NULL, major.by = NULL,
 					circos.axis(h = 0, major.at = major.at, labels = major.tick.labels, labels.cex = 0.3*par("cex"), labels.facing = "clockwise", major.tick.percentage = 0.2)
 				}
 				if(any(plotType %in% "labels")) {
-					circos.text(mean(xlim), 1.2, labels = sector.names[sector.index], cex = par("cex"), adj = c(0.5, 0))
+					circos.text(mean(xlim), 1.2, labels = sector.names[sector.index], cex = par("cex"), adj = c(0.5, 0), niceFacing = TRUE)
 				}
 			}
 		)
@@ -159,23 +169,26 @@ circos.genomicInitialize = function(data, sector.names = NULL, major.by = NULL,
 #
 # == param
 # -data A bed-file-like data frame or a list of data frames
-# -ylim If it is ``NULL``, the value will be calculated from data. If ``stack`` is set to ``TRUE``, the value is ignored.
-# -stack If ``data`` is a list of data frames or a data frame containing more than one numeric columns,
-#        whether to plot in a "stack" mode.
+# -ylim If it is ``NULL``, the value will be calculated from data. If ``stack`` is set to ``TRUE``, this value is ignored.
+# -stack whether to plot in a "stack" mode.
 # -numeric.column Columns of numeric values in ``data`` that will be used for plotting. 
 #                 If ``data`` is a data frame list, ``numeric.column`` should be either length of one or length of ``data``.
-#                 If value of ``numeric.column`` is not set, its value will be inferred from ``data``.
+#                 If value of ``numeric.column`` is not set, its value will depend on the structure of ``data``.
+#                 If ``data`` is a data frame, the default value for ``numeric.column`` is all the numeric column starting from the fourth column.
+#                 If ``data`` is a list of data frame, the default value for ``numeric.column`` is a vector which have the same length as ``data``
+#                 and the value in default ``numeric.column`` is the index of the first numeric column in corresponding data frame.
 # -panel.fun Self-defined function which will be applied on each sector. Please not it is different
 #            from that in `circos.trackPlotRegion`. In this function, there are two arguments (``region`` and ``value``) plus ``...``.
 #            In them, ``region`` is a two-column data frame with start positions and end positions in current genomic category (e.g. chromosome). 
 #            ``value`` is a data frame which is derived from ``data`` but excluding the first three columns. Rows in ``value`` correspond to 
 #            rows in ``region``. ``...`` is mandatory and is used to pass internal parameters to other functions. The definition of
-#            ``value`` will be different according to different input data and different settings, please refer to 'details' section and vignettes to detailed explaination.
+#            ``value`` will be different according to different input data (data frame or list of data frame) and different settings (stacked or not), 
+#            please refer to 'details' section and vignettes to detailed explanation.
 # -... Pass to `circos.trackPlotRegion`.
 #
 # == details
-# Similar as `circos.trackPlotRegion`, users can add customized graphics by ``panel.fun``, but the behavior of ``panel.fun``
-# would change depending on users' input data and ``stack`` setting.
+# Similar as `circos.trackPlotRegion`, users can add customized graphics by ``panel.fun``, but the behaviour of ``panel.fun``
+# will change depending on users' input data and ``stack`` setting.
 #
 # When ``data`` is a single data frame, ``region`` in ``panel.fun`` is a data frame containing the second and third column in ``data`` in 'current` genomic category (e.g. current chromosome).
 # ``value`` is also a data frame containing columns in ``data`` excluding the first three columns.
@@ -264,7 +277,7 @@ circos.genomicTrackPlotRegion = function(data = NULL, ylim = NULL, stack = FALSE
 	
 	args = formals(genomicPanelFun)
 	if(!(length(args) == 3 && names(args)[3] == "...")) {
-		stop("The `panel.fun` need a third argument `...` to pass specicial parameters to graphical functions.\n")
+		stop("The `panel.fun` need a third argument `...` to pass special parameters to graphical functions.\n")
 	}
 	
 	if(stack) {
@@ -387,7 +400,7 @@ circos.genomicTrackPlotRegion = function(data = NULL, ylim = NULL, stack = FALSE
 }
 
 # == title
-# Which data that panel.fun is using
+# Which data that ``panel.fun`` is using
 #
 # == param
 # -... Invisible arguments that users do not need to care
@@ -396,7 +409,9 @@ circos.genomicTrackPlotRegion = function(data = NULL, ylim = NULL, stack = FALSE
 # The function should only be put inside ``panel.fun`` when using `circos.genomicTrackPlotRegion`.
 #
 # If ``stack`` is set to ``TRUE`` in `circos.genomicTrackPlotRegion`, the returned value
-# indicates which stack the function will be applied to. If ``data`` is a list of data frames, the value
+# indicates which stack the function will be applied to.
+#
+# If ``data`` is a list of data frames, the value
 # indicates which data frame is being used. Please see the vignette to get a more clear explanation.
 getI = function(...) {
 	args = list(...)
@@ -412,13 +427,13 @@ getI = function(...) {
 # Add points to a plotting region, specifically for genomic graphics
 #
 # ==param
-# -region A data frame contains 2 column which correspond to start position and end position
+# -region A data frame contains 2 columns which correspond to start positions and end positions
 # -value  A data frame contains values and other information
 # -numeric.column Which column in ``value`` data frame should be taken as y-value.
 #                 If it is not defined, the whole numeric columns in ``value`` will be taken.
 # -sector.index Pass to `circos.points`
 # -track.index Pass to `circos.points`
-# -posTransform Self-defined functions to transform genomic positions, see `posTransform.default` for explaination
+# -posTransform Self-defined function to transform genomic positions, see `posTransform.default` for explanation
 # -col color of points. If there is only one numeric column, the length of ``col`` can be either one or number of rows of ``region``.
 #      If there are more than one numeric column, the length of ``col`` can be either one or number of numeric columns.
 #      Pass to `circos.points`
@@ -427,7 +442,7 @@ getI = function(...) {
 # -... Mysterious parameters
 #
 # == details
-# The function is usually put in ``panel.fun`` when using `circos.genomicTrackPlotRegion`.
+# The function is a low-level graphical function and usually is put in ``panel.fun`` when using `circos.genomicTrackPlotRegion`.
 circos.genomicPoints = function(region, value, numeric.column = NULL, 
 	sector.index = get.cell.meta.data("sector.index"),
     track.index = get.cell.meta.data("track.index"), posTransform = NULL, 
@@ -496,7 +511,7 @@ circos.genomicPoints = function(region, value, numeric.column = NULL,
 #                 If it is not defined, the whole numeric columns in ``value`` will be taken.
 # -sector.index Pass to `circos.lines`
 # -track.index Pass to `circos.lines`
-# -posTransform Self-defined functions to transform genomic positions, see `posTransform.default` for explaination
+# -posTransform Self-defined function to transform genomic positions, see `posTransform.default` for explaination
 # -col col of lines/areas. If there are more than one numeric column, the length of ``col`` can be either one or number of numeric columns.
 #      pass to `circos.lines`
 # -lwd Settings are similar as ``col``. Pass to `circos.lines`
@@ -512,7 +527,7 @@ circos.genomicPoints = function(region, value, numeric.column = NULL,
 # -... mysterious parameters
 #
 # == details
-# The function is usually put in ``panel.fun`` when using `circos.genomicTrackPlotRegion`.
+# The function is a low-level graphical function and usually is put in ``panel.fun`` when using `circos.genomicTrackPlotRegion`.
 circos.genomicLines = function(region, value, numeric.column = NULL, 
 	sector.index = get.cell.meta.data("sector.index"),
     track.index = get.cell.meta.data("track.index"), posTransform = NULL, 
@@ -626,7 +641,7 @@ circos.genomicLines = function(region, value, numeric.column = NULL,
 # -ybottom.column If ``ybottom`` is in ``value``, the index of the column
 # -sector.index Pass to `circos.rect`
 # -track.index Pass to `circos.rect`
-# -posTransform Self-defined functions to transform genomic positions, see `posTransform.default` for explaination
+# -posTransform Self-defined function to transform genomic positions, see `posTransform.default` for explaination
 # -col The length of ``col`` can be either one or number of rows of ``region``. Pass to `circos.rect`
 # -border Settings are similar as ``col``. Pass to `circos.rect`
 # -lty Settings are similar as ``col``. Pass to `circos.rect`
@@ -634,7 +649,7 @@ circos.genomicLines = function(region, value, numeric.column = NULL,
 # -... Mysterious parameters
 #
 # == details
-# The function is usually put in ``panel.fun`` when using `circos.genomicTrackPlotRegion`.
+# The function is a low-level graphical function and usually is put in ``panel.fun`` when using `circos.genomicTrackPlotRegion`.
 circos.genomicRect = function(region, value = NULL, 
 	ytop = NULL, ybottom = NULL, ytop.column = NULL, ybottom.column = NULL,
 	sector.index = get.cell.meta.data("sector.index"),
@@ -731,22 +746,24 @@ circos.genomicRect = function(region, value = NULL,
 #                 If it is not defined, only the first numeric columns in ``value`` will be taken.
 # -sector.index Pass to `circos.rect`
 # -track.index Pass to `circos.rect`
-# -posTransform Self-defined functions to transform genomic positions, see `posTransform.default` for explaination
+# -posTransform Self-defined function to transform genomic positions, see `posTransform.default` for explanation
 # -facing Passing to `circos.text`. Settings are similar as ``col`` 
+# -niceFacing   Should the facing of text be adjusted to fit human eyes?
 # -direction Deprecated, use ``facing`` instead. 
 # -adj Pass to `circos.text`. Settings are similar as ``col``
 # -cex Pass to `circos.text`. Settings are similar as ``col``
 # -col Pass to `circos.text`. The length of ``col`` can be either one or number of rows of ``region``.
 # -font Pass to `circos.text`. Settings are similar as ``col``
+# -padding pass to ``posTransform`` if it is set as `posTransform.text`
 # -... Mysterious parameters
 #
 # == details
-# The function is usually put in ``panel.fun`` when using `circos.genomicTrackPlotRegion`.
+# The function is a low-level graphical function and usually is put in ``panel.fun`` when using `circos.genomicTrackPlotRegion`.
 circos.genomicText = function(region, value, y = NULL, labels = NULL, labels.column = NULL,
 	numeric.column = NULL, sector.index = get.cell.meta.data("sector.index"), 
 	track.index = get.cell.meta.data("track.index"), posTransform = NULL, 
-	direction = NULL, facing = "inside",
-	adj = par("adj"), cex = 1, col = "black", font = par("font"), ...) {
+	direction = NULL, facing = "inside", niceFacing = FALSE,
+	adj = par("adj"), cex = 1, col = "black", font = par("font"), padding = 0, ...) {
 	
 	if(!is.null(direction)) {
 		facing = direction
@@ -800,10 +817,6 @@ circos.genomicText = function(region, value, y = NULL, labels = NULL, labels.col
 		}
 	}
 
-	if(!is.null(posTransform)) {
-		region = posTransform(region)
-	}
-
 	if(is.null(numeric.column)) {
 		numeric.column = which(as.logical(sapply(value, is.numeric)))
 		if(length(numeric.column) == 0) {
@@ -815,7 +828,18 @@ circos.genomicText = function(region, value, y = NULL, labels = NULL, labels.col
 	if(length(numeric.column) > 1) {
 		stop("You can only have one numeric column.\n")
 	}
-
+	
+	if(!is.null(posTransform)) {
+	
+		# check settings when it is text-specific transformation
+		if(identical(posTransform, posTransform.text)) {
+			if(! facing %in% c("clockwise", "reverse.clockwise")) {
+				stop("Only support `facing` in c('clockwise', 'reverse.clockwise') if `posTransform` is `posTransform.text`.\n")
+			}
+		}
+		region = posTransform(region, value[[ numeric.column ]], value[[labels.column]], cex, font, padding = padding)
+	}
+	
 	nc = length(numeric.column)
 
 	col = .normalizeGraphicalParam(col, nc, nr, "col")
@@ -823,7 +847,7 @@ circos.genomicText = function(region, value, y = NULL, labels = NULL, labels.col
 	font = .normalizeGraphicalParam(font, nc, nr, "font")
 
 	circos.text( (region[[1]] + region[[2]])/2, value[[ numeric.column ]], value[[labels.column]],
-		facing = facing, adj = adj, cex = cex, col = col, font = font,
+		facing = facing, niceFacing = niceFacing, adj = adj, cex = cex, col = col, font = font,
 		sector.index = sector.index, track.index = track.index )
 
 }
@@ -846,8 +870,19 @@ circos.genomicText = function(region, value, y = NULL, labels = NULL, labels.col
 #
 # == details
 # Of course, number of rows should be same in ``region1`` and ``region2``.
+#
+# If you want to have more controls on links, please use `circos.link` directly.
 circos.genomicLink = function(region1, region2, 
-	rou = get.track.end.position(get.current.track.index()), rou1 = rou, rou2 = rou,
+	rou = {tracks = get.all.track.index()
+	       if(length(tracks) == 0) {
+		       1
+		   } else {
+		       n = length(tracks)
+		       get.cell.meta.data("cell.bottom.radius", track.index = tracks[n]) - 
+			     get.cell.meta.data("track.margin", track.index = tracks[n])[1] - 
+			     circos.par("track.margin")[2]
+		   }
+    }, rou1 = rou, rou2 = rou,
     col = "black", lwd = par("lwd"), lty = par("lty"), border = NA, ...) {
 	
 	region1 = normalizeToDataFrame(region1)
@@ -900,24 +935,26 @@ circos.genomicLink = function(region1, region2,
 # -data A data frame containing genomic data
 # -track.height Height of the track
 # -posTransform Genomic position transformation function, see `posTransform.default` for an example.
-# -horizontalLine Whether to draw horizontal lines which indicate width of each region
+# -horizontalLine Whether to draw horizontal lines which indicate region width 
 # -track.margin Margin of tracks
-# -type Type of the transformation. ``default`` means position transformed track are located inside 
-#       and ``reverse`` means position transformed track are located outside.
-# -col Color of lines, can be length of one or length of nrow of ``data``
+# -direction Type of the transformation. ``inside`` means position transformed track are located inside 
+#       and ``outside`` means position transformed track are located outside.
+# -col Color of lines, can be length of one or nrow of ``data``
 # -lwd Width of lines
 # -lty Style of lines
+# -... pass to `circos.trackPlotRegion`
 #
 # == details
 # There is one representative situation when such position transformation needs to be applied. 
 # For example, there are two sets of regions in a chromosome in which regions in one set regions are
 # quite densely to each other and regions in other set are far from others. Heatmap or text is going
-# to be drawn on the next track. If there is no position transformtion, heatmap or text for those
+# to be drawn on the next track. If there is no position transformation, heatmap or text for those
 # dense regions would be overlapped and hard to identify, also ugly to visualize. Thus, a way
 # to transform original positions to new positions would help for the visualization. 
 circos.genomicPosTransformLines = function(data, track.height = 0.1, posTransform = NULL, 
 	horizontalLine = c("none", "top", "bottom", "both"), track.margin = c(0, 0),
-	type = c("default", "reverse"), col = "black", lwd = par("lwd"), lty = par("lty")) {
+	direction = c("inside", "outside"), col = "black", lwd = par("lwd"),
+    lty = par("lty"), ...) {
 	
 	horizontalLine = match.arg(horizontalLine)[1]
 
@@ -941,54 +978,63 @@ circos.genomicPosTransformLines = function(data, track.height = 0.1, posTransfor
 	o.track.margin = circos.par("track.margin")
 	circos.par(track.margin = track.margin)
 	
-	type = match.arg(type)[1]
+	if(direction[1] == "default") direction = "outside"
+	if(direction[1] == "reverse") direction = "inside"
+	direction = match.arg(direction)[1]
 	
-	if(type == "default") {
-		circos.trackPlotRegion(data[[1]], ylim = c(0, 1), bg.border = NA, track.height = track.height, panel.fun = function(x, y) {
+	if(direction == "inside") {
+		circos.genomicTrackPlotRegion(data, ylim = c(0, 1), bg.border = NA, track.height = track.height, panel.fun = function(region, value, ...) {
 			chr = get.current.chromosome()
 			l = data[[1]] == chr
-			region_subset = data[l, , drop = FALSE]
-			if(is.null(posTransform)) {
-				region_new_subset = region_subset
+			if(!is.null(posTransform)) {
+				if(is.function(posTransform)) {
+					region_new = posTransform(region)
+				} else if(is.call(posTransform)) {
+					region_new = eval(posTransform)
+				}
 			} else {
-				region_new_subset = cbind(region_subset[[1]], posTransform(region_subset[2:3]))
+				region_new  = region
 			}
 			
-			for(i in seq_len(nrow(region_subset))) {
+			for(i in seq_len(nrow(region))) {
 				if(horizontalLine == "both" || horizontalLine == "top") {
-					circos.lines(c(region_subset[i, 2], region_subset[i, 3]), c(1, 1), col = col[l][i], lwd = lwd[l][i], lty = lty[l][i])
+					circos.lines(c(region[i, 1], region[i, 2]), c(1, 1), col = col[l][i], lwd = lwd[l][i], lty = lty[l][i])
 				}
 				if(horizontalLine == "both" || horizontalLine == "bottom") {
-					circos.lines(c(region_new_subset[i, 2], region_new_subset[i, 3]), c(0, 0), col = col[l][i], lwd = lwd[l][i], lty = lty[l][i])
+					circos.lines(c(region[i, 1], region[i, 2]), c(0, 0), col = col[l][i], lwd = lwd[l][i], lty = lty[l][i])
 				}
-				mid = (region_subset[i, 2] + region_subset[i, 3])/2
-				mid_new = (region_new_subset[i, 2] + region_new_subset[i, 3])/2
+				mid = (region[i, 1] + region[i, 2])/2
+				mid_new = (region_new[i, 1] + region_new[i, 2])/2
 				circos.lines(c(mid, mid, mid_new, mid_new), c(1, 2/3, 1/3, 0), col = col[l][i], lwd = lwd[l][i], lty = lty[l][i])
 			}
-		})
+		}, ...)
 	} else {
-		circos.trackPlotRegion(data[[1]], ylim = c(0, 1), bg.border = NA, track.height = track.height, panel.fun = function(x, y) {
+		circos.genomicTrackPlotRegion(data, ylim = c(0, 1), bg.border = NA, track.height = track.height, panel.fun = function(region, value, ...) {
 			chr = get.current.chromosome()
 			l = data[[1]] == chr
 			region_subset = data[l, , drop = FALSE]
-			if(is.null(posTransform)) {
-				region_new_subset = region_subset
+			if(!is.null(posTransform)) {
+				if(is.function(posTransform)) {
+					region_new = posTransform(region)
+				} else if(is.call(posTransform)) {
+					region_new = eval(posTransform)
+				}
 			} else {
-				region_new_subset = cbind(region_subset[[1]], posTransform(region_subset[2:3]))
+				region_new  = region
 			}
 			
-			for(i in seq_len(nrow(region_subset))) {
+			for(i in seq_len(nrow(region))) {
 				if(horizontalLine == "both" || horizontalLine == "bottom") {
-					circos.lines(c(region_subset[i, 2], region_subset[i, 3]), c(0, 0), col = col[l][i], lwd = lwd[l][i], lty = lty[l][i])
+					circos.lines(c(region[i, 1], region[i, 2]), c(0, 0), col = col[l][i], lwd = lwd[l][i], lty = lty[l][i])
 				}
 				if(horizontalLine == "both" || horizontalLine == "top") {
-					circos.lines(c(region_new_subset[i, 2], region_new_subset[i, 3]), c(1, 1), col = col[l][i], lwd = lwd[l][i], lty = lty[l][i])
+					circos.lines(c(region[i, 1], region[i, 2]), c(1, 1), col = col[l][i], lwd = lwd[l][i], lty = lty[l][i])
 				}
-				mid = (region_subset[i, 2] + region_subset[i, 3])/2
-				mid_new = (region_new_subset[i, 2] + region_new_subset[i, 3])/2
+				mid = (region[i, 1] + region[i, 2])/2
+				mid_new = (region_new[i, 1] + region_new[i, 2])/2
 				circos.lines(c(mid, mid, mid_new, mid_new), c(0, 1/3, 2/3, 1), col = col[l][i], lwd = lwd[l][i], lty = lty[l][i])
 			}
-		})
+		}, ...)
 	}
 	
 	circos.par(track.margin = o.track.margin)
@@ -1055,7 +1101,7 @@ circos.genomicDensity = function(data, ylim.force = FALSE, window.size = NULL, o
 	s = sapply(get.all.sector.index(), function(si) get.cell.meta.data("xrange", sector.index = si))
 	if(is.null(window.size)) {
 		window.size = 10^nchar(sum(s))/1000  # around 100 major ticks
-		cat(window.size, "is choosen as the window size.\n")
+		#cat(window.size, "is choosen as the window size.\n")
 	}
 	
 	df = vector("list", length = length(data))
@@ -1126,26 +1172,6 @@ genomicDensity = function(region, window.size = 10000000, overlap = TRUE) {
 	return(res)
 }
 
-# == title
-# Genomic position transformation function
-#
-# == param
-# -region Genomic positions at a single chromosome. It is a data frame with two
-#     columns which are start position and end position.
-#
-# == details
-# The default position transformation functions transforms position to be equally distributed
-# along the chromosome. If users want to define their own transformation function, the requirement
-# is that the returned value should be a data frame with two columns: transformed start position
-# and transformed end position. The returned value should have same number of rows as the input one.
-#
-# For details why need to use position transformation, please refer to `circos.genomicPosTransformLines`.
-posTransform.default = function(region) {
-	xlim = get.cell.meta.data("xlim")
-	segment = seq(xlim[1], xlim[2], length.out = nrow(region) + 1)
-	return(data.frame(start = segment[-length(segment)], end = segment[-1]))
-}
-
 # == title
 # Highlight a chromosome
 #
@@ -1163,7 +1189,7 @@ posTransform.default = function(region) {
 # You may use `circos.info` to find out index for all tracks.
 #
 # The function calls `draw.sector`.
-highlight.chromosome = function(chr, track.index = seq_len(get.max.track.index()), 
+highlight.chromosome = function(chr, track.index = get.all.track.index(), 
 	col = "#FF000040", border = NA, lwd = par("lwd"), lty = par("lty"),
 	padding = c(0, 0, 0, 0)) {
 	
@@ -1171,8 +1197,8 @@ highlight.chromosome = function(chr, track.index = seq_len(get.max.track.index()
 		stop("`chr` can only be length 1.\n")
 	}
 	
-	max.track.index = get.max.track.index()
-	if(!all(track.index %in% seq_len(max.track.index))) {
+	tracks = get.all.track.index()
+	if(!all(track.index %in% tracks)) {
 		stop("`track.index` contains index that does not belong to available sectors.\n")
 	}
 	
@@ -1286,7 +1312,7 @@ normalizeToDataFrame = function(data) {
 # This is high-level graphical function, which mean, it will create a new track.
 #
 # Rainfall plot can be used to visualize distribution of regions. On the plot, y-axis
-# corresponds to the distance to neighbour regions. So if there is a drop-down on
+# corresponds to the distance to neighbour regions (log-based). So if there is a drop-down on
 # the plot, it means there is a cluster of regions at that area.
 #
 # On the plot, y-axis are log10-transformed.
@@ -1359,4 +1385,124 @@ rainfallTransform = function(region, mode = c("min", "max", "mean")) {
 	dist = ifelse(dist < 0, 0, dist)
 	
 	return(data.frame(start = region[, 1], end = region[, 2], dist = dist))
-}
\ No newline at end of file
+}
+
+# == title
+# Genomic position transformation function
+#
+# == param
+# -region Genomic positions at a single chromosome. It is a data frame with two
+#     columns which are start position and end position.
+# -... black hole to eat junk arguments.
+#
+# == details
+# The default position transformation functions transforms position to be equally distributed
+# along the chromosome. If users want to define their own transformation function, the requirement
+# is that the returned value should be a data frame with two columns: transformed start position
+# and transformed end position. The returned value should have same number of rows as the input one.
+#
+# For details why need to use position transformation, please refer to `circos.genomicPosTransformLines`.
+posTransform.default = function(region, ...) {
+	xlim = get.cell.meta.data("xlim")
+	segment = seq(xlim[1], xlim[2], length.out = nrow(region) + 1)
+	return(data.frame(start = segment[-length(segment)], end = segment[-1]))
+}
+
+# == title
+# Genomic position transformation function specifically for text
+#
+# == param
+# -region Genomic positions at a single chromosome. It is a data frame with two
+#     columns which are start position and end position.
+# -y positions of texts
+# -labels text labels
+# -cex  text size
+# -font  text font style
+# -sector.index sector index
+# -track.index track index
+# -padding padding of text
+# -... black hole to eat junk arguments.
+#
+# == details
+# This position transformation function is designed specifically for text.
+# Under the transformation, texts will be as close as possible to the original positions.
+posTransform.text = function(region, y, labels, cex = 1, font = par("font"),
+	sector.index = get.cell.meta.data("sector.index"),
+	track.index = get.cell.meta.data("track.index"), padding = 0, ...) {
+	
+	if(length(y) == 1) y = rep(y, nrow(region))
+	if(length(labels) == 1) labels = rep(labels, nrow(region))
+	
+	od = order(region[[1]])
+	region = region[od, ]
+	y = y[od]
+	labels = labels[od]
+
+	text_height = strheight(labels, cex = cex, font = font)*(1+padding)
+	
+	d = circlize( (region[[1]] + region[[2]])/2, y, sector.index = sector.index, track.index = track.index)
+	alpha1 = d[, "theta"] + as.degree(atan(text_height/2/d[, "rou"]))
+	alpha2 = d[, "theta"] - as.degree(atan(text_height/2/d[, "rou"]))
+	x1 = reverse.circlize(alpha1, d[, "rou"], sector.index = sector.index, track.index = track.index)[, "x"]
+	x2 = reverse.circlize(alpha2, d[, "rou"], sector.index = sector.index, track.index = track.index)[, "x"]
+	
+	xlim = get.cell.meta.data("xlim", sector.index = sector.index, track.index = track.index)
+	
+	x1_new = x1
+	x2_new = x2
+	l = x2 - x1 >= xlim[2] - xlim[1]; x1_new[l] = xlim[1]; x2_new[l] = xlim[2]
+	l = x1 < xlim[1]; x1_new[l] = xlim[1]; x2_new[l] = x2[l] + xlim[1] - x1[l]
+	l = x2 > xlim[2]; x1_new[l] = x1[l] - (x2[l] - xlim[2]); x2_new[l] = xlim[2]
+	return(smartAlign(x1_new, x2_new, xlim = xlim))
+}
+
+# x1 should be sorted
+smartAlign = function(x1, x2, xlim) {
+	
+	ncluster.before = -1
+	ncluster = length(x1)
+	while(ncluster.before != ncluster) {
+		ncluster.before = ncluster
+		cluster = rep(0, length(x1))
+		i_cluster = 1
+		cluster[1] = i_cluster
+		for(i in seq_along(x1)[-1]) {
+			# overlap with previous one
+			if(x1[i] <= x2[i-1]) {
+				cluster[i] = i_cluster
+			} else {
+				i_cluster = i_cluster + 1
+				cluster[i] = i_cluster
+			}
+		}
+		ncluster = length(unique(cluster))
+		
+		if(ncluster.before == ncluster) break
+		
+		# tile intervals in each cluster and re-assign x1 and x2
+		new_x1 = numeric(length(x1))
+		new_x2 = numeric(length(x2))
+		for(i_cluster in unique(cluster)) {
+			index = which(cluster == i_cluster)
+			total_len = sum(x2[index] - x1[index])
+			mid = (min(x1[index]) + max(x2[index]))/2
+			if(total_len > xlim[2] - xlim[1]) {
+				tp = seq(xlim[1], xlim[2], length = length(index) + 1)
+			} else if(mid - total_len/2 < xlim[1]) {
+				tp = seq(xlim[1], xlim[1] + total_len, length = length(index) + 1)
+			} else if(mid + total_len/2 > xlim[2]) {
+				tp = seq(xlim[2] - total_len, xlim[2], length = length(index) + 1)
+			} else {
+				tp = seq(mid - total_len/2, mid + total_len/2, length = length(index)+1)
+			}
+			new_x1[index] = tp[-length(tp)]
+			new_x2[index] = tp[-1]
+		}
+		
+		x1 = new_x1
+		x2 = new_x2
+	}
+	
+	return(data.frame(start = x1, end = x2))
+}
+
