@@ -2,66 +2,82 @@
 # Create plotting regions for a whole track
 #
 # == param
-# -factors      Factors which represent the categories of data, if is ``NULL``, 
-#               then it is the whole sector index.
-# -x            Data on the x-axis
-# -y            Data on the y-axis
-# -ylim         Range of data on the y-axis
-# -force.ylim   Whether to force all cells in the track to share the same ``ylim``
-# -track.index  Index for the track which is goning to be updated. Setting it to ``NULL`` means
-#               creating the plotting regions in the next newest track.
-# -track.height Height of the track. It is the percentage to the radius of the unit circls.
-#               If to update a track, this argument is disabled.
-# -bg.col       Background color for the plotting regions
-# -bg.border    Color for the boder of the plotting regions
-# -bg.lty       Line style for the border of the plotting regions
-# -bg.lwd       Line width for the border of the plotting regions
-# -panel.fun    Panel function to draw figures in each cell, see "details" section
-#               and vignette for explaination.
+# -factors      Factors which represent categories of data, if it is ``NULL``, 
+#               then it uses the whole sector index.
+# -x            Data on x-axis. It is only used if ``panel.fun`` is set.
+# -y            Data on y-axis
+# -ylim         Range of data on y-axis
+# -force.ylim   Whether to force all cells in the track to share the same ``ylim``. Normally,
+#               all cells on a same track should have same ``ylim``.
+# -track.index  Index for the track which is going to be created/updated. If the specified track has already
+#               been created, this function just updated corresponding track with new plot. If the specified track
+#               is ``NULL`` or has not been created, this function just created it. Note the value for this
+#               argument should not exceed maximum track index plus 1.
+# -track.height Height of the track. It is the percentage to the radius of the unit circles.
+#               If updating a track (with proper ``track.index`` value), this argument is ignored.
+# -track.margin only affect current track
+# -cell.padding only affect current track
+# -bg.col       Background color for the plotting regions. It can be vector which has the same length of sectors.
+# -bg.border    Color for the border of the plotting regions. It can be vector which has the same length of sectors.
+# -bg.lty       Line style for the border of the plotting regions. It can be vector which has the same length of sectors.
+# -bg.lwd       Line width for the border of the plotting regions. It can be vector which has the same length of sectors.
+# -panel.fun    Panel function to add graphics in each cell, see "details" section
+#               and vignette for explanation.
 #
 # == details
 # This function pretends to be a high-level plotting function, which means, 
-# you must first call this function to create a plotting region, then those
-# low-level-style plotting function such as `circos.points`, `circos.lines` can be
+# you must first call this function to create plotting regions, then those
+# low-level graphical function such as `circos.points`, `circos.lines` can be
 # applied.
 #
 # It has two different usages. First, it can create a complete track which among several
-# sectors. Because currently it does not support creating single cell since it would
-# make the layout disordered, this is the only way to create the plotting regions.
+# sectors. Because currently it does not support creating single cell since it will
+# make the layout disordered, this is the only way to create plotting regions.
 #
 # Currently, all the cells that are created in a same track sharing same height, which means,
-# there is no cell has longer height than others.
+# there is no cell has larger height than others.
 #
 # Since limitation for values on x-axis has already been defined by `circos.initialize`, only
 # limitation for values on y-axis should be specified in this function. The ``x`` argument is only
-# used if you set ``panel.fun``. There are two ways to identify the limitation for values on y-axies either by ``y``
-# or ``ylim``. If ``y`` is set, it must has the same length as ``factors`` and the ylim or each cell is calculated
+# used if you set ``panel.fun``. There are two ways to identify the limitation for values on y-axes either by ``y``
+# or ``ylim``. If ``y`` is set, it must has the same length as ``factors`` and the ``ylim`` for each cell is calculated
 # from y values. Also, the ylim can be specified from ``ylim`` which can be a two-element vector or a matrix which
 # has two columns and the number of rows is the same as the length of the levels of the factors.
 #
 # If there is no enough space for the new track or the new track has overlap with other tracks,
 # there will be an error.
 #
-# ``panel.fun`` provides a convinient way to draw figures in each cell when initializing the 
-# track. The self-defined function need two arguments: ``x`` and ``y`` which is the data points
+# ``panel.fun`` provides a convenient way to add graphics in each cell when initializing the 
+# tracks. The self-defined function need two arguments: ``x`` and ``y`` which correspond to the data points
 # in the current cell. `circos.trackPlotRegion` creates plotting regions one by one on the track and
-# ``panel.fun`` draw graphs in the 'current' cell after the plotting region of a certain cell has been
+# ``panel.fun`` adds graphics in the 'current' cell after the plotting region for a certain cell has been
 # created. See vignette for examples of how to use this feature.
 #
-# If ``factors`` does not cover all sectors which is going to be initialized, the cells in remaining unselected
+# If ``factors`` does not cover all sectors, the cells in remaining unselected
 # sectors would also be created but without drawing anything. The ``ylim`` for these cells
 # are the same as that in the latest created cell.
 #
 # Second, it can update a already-created track if the index for the track
-# is specified. If the index is one larger than the largest current track index, it in fact
-# creates the new track. If updating an existed track, those parameters related to the position
+# is specified. If the index is one larger than the largest track index, it in fact
+# creates the new track. If updating an existed track, those parameters related to the position (such as track height and track margin)
 # of the plotting region can not be changed.
 circos.trackPlotRegion = function(factors = NULL, x = NULL, y = NULL, ylim = NULL,
     force.ylim = TRUE, track.index = NULL,
 	track.height = circos.par("default.track.height"),
+	track.margin = circos.par("track.margin"),
+	cell.padding = circos.par("cell.padding"),
     bg.col = NA, bg.border = "black", bg.lty = par("lty"), bg.lwd = par("lwd"),
     panel.fun = function(x, y) {NULL}) {
     
+    if(!is.circos.initialized()) {
+    	stop("Your circos plot has not been initialized yet!\n")
+    }
+	
+	o.track.margin = circos.par("track.margin")
+	o.cell.padding = circos.par("cell.padding")
+	circos.par(track.margin = track.margin)
+	circos.par(cell.padding = cell.padding)
+	
 	# if there is no factors, default are all the available factors
 	if(is.null(factors)) {
 		factors = get.all.sector.index()
@@ -86,20 +102,29 @@ circos.trackPlotRegion = function(factors = NULL, x = NULL, y = NULL, ylim = NUL
         stop(paste("Cannot find these categories in existed sectors:", paste(setdiff.factors, collapse = ", "), ".\n", sep = ""))
     }
     
+	tracks = get.all.track.index()
+	last.track.index = ifelse(length(tracks), tracks[length(tracks)], 0)
+	flag_createNewTrack = 0
     if(is.null(track.index)) {
         # new track should inside the most recently created track
-        last.track.index = get.max.track.index()
         set.current.track.index(last.track.index + 1)
         track.index = get.current.track.index()
-    } else {   # update an existed track
-		if(track.index > get.max.track.index() + 1) {
-			stop(paste("Wrong track index: it should be no more than ", get.max.track.index(), "\n", sep = ""))
+		flag_createNewTrack = 1
+    } else if(track.index == last.track.index + 1) {
+		# if the track.index is next to the most recently created track
+		set.current.track.index(track.index)
+        track.index = get.current.track.index()
+		flag_createNewTrack = 1
+	} else {   
+		if(track.index > last.track.index + 1) {
+			stop(paste("Wrong track index: it should be no more than ", last.track.index + 1, "\n", sep = ""))
 		}
-		if(track.index <= get.max.track.index()) {
-			#if(! is.null(track.height)) {
-			#	warning("You are updating an existed track, the `track.height` is not used.\n")
-			#}
-			track.height = get.cell.data(factors[1], track.index)$track.height
+		# update an existed track
+		if(track.index <= tracks[length(tracks)]) {
+			# ignore track.height from args
+			track.height = get.cell.meta.data("track.height", sector.index = factors[1], track.index = track.index)
+			# ignore track.margin 
+			circos.par("track.margin" = get.cell.meta.data("track.margin", sector.index = factors[1], track.index = track.index))
 		}
         if(is.null(ylim) && is.null(y)) {
             for(sid in get.all.sector.index()) {
@@ -132,15 +157,21 @@ circos.trackPlotRegion = function(factors = NULL, x = NULL, y = NULL, ylim = NUL
 		}
 	}
 	
-	if(track.index <= get.max.track.index()) {  # if just update a whole track
-		track.start = get.cell.data(factors[1], track.index)$track.start
-	} else {
-		track.start = get.track.end.position(track.index - 1) - circos.par("track.margin")[2]
-    }
+	if(flag_createNewTrack) {
+		if(track.index == 1) {
+			track.start = 1 - circos.par("track.margin")[2]
+		} else {
+			track.start = get.cell.meta.data("cell.bottom.radius", track.index = track.index - 1) - 
+			              get.cell.meta.data("track.margin", track.index = track.index - 1)[1] -
+						  circos.par("track.margin")[2]
+		}
+    } else {
+		track.start = get.cell.meta.data("cell.top.radius", track.index = track.index)
+	}
 	
     # check whether there is enough space for the new track and whether the new space
     # overlap with other tracks. Only for creatation mode.
-	if(track.index > get.max.track.index()) {
+	if(flag_createNewTrack) {
 		check.track.position(track.index, track.start, track.height)
     }
 	
@@ -151,7 +182,7 @@ circos.trackPlotRegion = function(factors = NULL, x = NULL, y = NULL, ylim = NUL
 		} else if(is.matrix(ylim) && ncol(ylim) == 2 && nrow(ylim) == length(le)) {
 		
 		} else {
-			stop("Wrong `ylim`.\n")
+			stop("Wrong `ylim` format.\n")
 		}
     } 
         
@@ -204,9 +235,9 @@ circos.trackPlotRegion = function(factors = NULL, x = NULL, y = NULL, ylim = NUL
 		}
 	}
 	
-    # After the track has been created, the default tract starting position is set
-    # just next to the most recently created track
-    set.track.end.position(track.index, track.start - track.height - circos.par("track.margin")[1])
+	circos.par(track.margin = o.track.margin)
+	circos.par(cell.padding = o.cell.padding)
+	
     return(invisible(NULL))
 
 }
@@ -218,12 +249,12 @@ circos.trackPlotRegion = function(factors = NULL, x = NULL, y = NULL, ylim = NUL
 # -sector.index Index for the sector
 # -track.index  Index for the track
 # -bg.col       Background color for the plotting region
-# -bg.border    Color for the boder of the plotting region
+# -bg.border    Color for the border of the plotting region
 # -bg.lty       Line style for the border of the plotting region
 # -bg.lwd       Line width for the border of the plotting region
 #
 # == details
-# You can update an existed cell by this function by erasing the contents in the plotting regions.
+# You can update an existed cell by this function by erasing all the graphics.
 # But the ``xlim`` and ``ylim`` inside the cell still remains unchanged. 
 #
 # Note if you use `circos.trackPlotRegion` to update an already created track, you can re-define ``ylim`` in these cells.
@@ -262,12 +293,10 @@ circos.createPlotRegion = function(track.start, track.height = circos.par("defau
 	
 	cell.padding = circos.par("cell.padding")
 
-	xlim = numeric(2)
-	xlim[1] = cell.xlim[1] + (cell.xlim[2] - cell.xlim[1]) / (sector.data["start.degree"] - sector.data["end.degree"]) *cell.padding[2]
-	xlim[2] = cell.xlim[2] - (cell.xlim[2] - cell.xlim[1]) / (sector.data["start.degree"] - sector.data["end.degree"]) *cell.padding[4]
+	xlim = c(sector.data["min.data"], sector.data["max.data"])
 	
 	if(cell.padding[1] + cell.padding[3] >= track.height) {
-		stop("Sumation of cell padding on y-direction are larger than the height of the cells.\n")
+		stop("Summation of cell padding on y-direction are larger than the height of the cells.\n")
 	}
 	
 	yl = numeric(2)
@@ -306,20 +335,20 @@ circos.createPlotRegion = function(track.start, track.height = circos.par("defau
 # -y            Data points on y-axis
 # -sector.index Index for the sector
 # -track.index  Index for the track
-# -pch          Points type
-# -col          Points color
-# -cex          Points size
+# -pch          Point type
+# -col          Point color
+# -cex          Point size
 #
 # == details
-# This function can only add points in a specified cell. Pretending a low-level plotting 
-# function, it can only be applied in plottting region which has been created.
+# This function can only add points in one specified cell. Pretending a low-level plotting 
+# function, it can only be applied in plotting region which has been created.
 #
 # You can think the function as the normal `graphics::points`
 # function, just adding points in the plotting region. The position of
 # plotting region is identified by ``sector.index`` and ``track.index``, if they are not
-# specified, they are in current sector and current track.
+# specified, they are in 'current' sector and 'current' track.
 #
-# Data points out of the plotting region will be drawed, but with a warning message.
+# Data points out of the plotting region will also be added, but with warning messages.
 #
 # Other graphics parameters which are available in the function are ``pch``, ``col``
 # and ``cex`` which have same meaning as those in the `graphics::par`.
@@ -328,11 +357,11 @@ circos.points = function(x, y, sector.index = get.cell.meta.data("sector.index")
     pch = par("pch"), col = par("col"), cex = par("cex")) {
     
     if(!has.cell(sector.index, track.index)) {
-        stop("'circos.points' can only be used after the plotting region been created\n")
+        stop("'circos.points' can only be used after the plotting region has been created\n")
     }
 	
 	if(length(x) != length(y)) {
-		stop("length of x and y differ.\n")
+		stop("Length of x and y differ.\n")
 	}
 
     # whether the points that are out of the plotting region.
@@ -352,17 +381,16 @@ circos.points = function(x, y, sector.index = get.cell.meta.data("sector.index")
 # -x            Data points on x-axis
 # -y            Data points on y-axis
 # -track.index  Index for the track
-# -pch          Points type
-# -col          Points color
-# -cex          Points size
+# -pch          Point type
+# -col          Point color
+# -cex          Point size
 #
 # == details
 # The function adds points in multiple cells by first splitting data into several parts in which
-# each part corresponds to one factor (sector index) and then add points in cells corresponding
-# to the part of data by calling `circos.points`.
+# each part corresponds to one factor (sector index) and then adding points in each cell by calling `circos.points`.
 #
-# Length of ``pch``, ``col`` and ``cex`` can be one, length of levels of the factors and length of 
-# factors. All length will be recycled to the length of factors respectively.
+# Length of ``pch``, ``col`` and ``cex`` can be one, length of levels of the factors or length of 
+# factors.
 #
 # This function can be replaced by a ``for`` loop containing `circos.points`.
 circos.trackPoints = function(factors = NULL, x, y, track.index = get.cell.meta.data("track.index"),
@@ -420,21 +448,21 @@ circos.trackPoints = function(factors = NULL, x, y, track.index = get.cell.meta.
 # -type         line type, similar as ``type`` argument in `graphics::lines`, but only in ``c("l", "o", "h", "s")``
 # -straight     whether draw straight lines between points
 # -area         whether to fill the area below the lines. If it is set to ``TRUE``, ``col`` controls the filled color
-#               in the area and ``border`` controls the color of the line.
+#               in the area and ``border`` controls color of the line.
 # -area.baseline deprecated, use ``baseline`` instead.
-# -baseline     the base line to draw area below lines, default is the minimal of y-range (most bottom). It can be a string or number.
-#               If a string, it should be one of ``bottom`` and ``top``.
+# -baseline     the base line to draw areas. By default it is the minimal of y-range (bottom). It can be a string or a number.
+#               If a string, it should be one of ``bottom`` and ``top``. This argument also works if ``type`` is set to ``h``.
 # -border       color for border of the area
-# -pt.col       if ``type`` is "o", points color
-# -cex          if ``type`` is "o", points size
-# -pch          if ``type`` is "o", points type
+# -pt.col       if ``type`` is "o", point color
+# -cex          if ``type`` is "o", point size
+# -pch          if ``type`` is "o", point type
 #
 # ==details
 # Normally, straight lines in the Cartesian coordinate have to be transformed into curves in the circos layout.
 # But if you do not want to do such transformation you can use this function just drawing straight
 # lines between points by setting ``straight`` to ``TRUE``.
 #
-# Draw areas below lines can help to identify the direction of y-axis in cells (since it is a circle). This can be fullfilled by specifying
+# Draw areas below lines can help to identify the direction of y-axis in cells (since it is a circle). This can be done by specifying
 # ``area`` to ``TURE``.
 circos.lines = function(x, y, sector.index = get.cell.meta.data("sector.index"),
     track.index = get.cell.meta.data("track.index"),
@@ -448,7 +476,7 @@ circos.lines = function(x, y, sector.index = get.cell.meta.data("sector.index"),
 	}
 	
 	if(length(x) != length(y)) {
-		stop("length of x and y differ.\n")
+		stop("Length of x and y differ.\n")
 	}
 	
 	if(baseline == "bottom") {
@@ -539,7 +567,7 @@ circos.lines = function(x, y, sector.index = get.cell.meta.data("sector.index"),
 # -area         whether to fill the area below the lines. If it is set to ``TRUE``, ``col`` controls the filled color
 #               in the area and ``border`` controls the color of the line.
 # -area.baseline deprecated, use ``baseline`` instead.
-# -baseline the base line to draw area under lines, default is ``NA`` which means the baseline for each cell would be calculated seperately
+# -baseline the base line to draw area, pass to `circos.lines`.
 # -border       color for border of the area
 # -pt.col       if ``type`` is "o", points color
 # -cex          if ``type`` is "o", points size
@@ -547,13 +575,12 @@ circos.lines = function(x, y, sector.index = get.cell.meta.data("sector.index"),
 #
 # == details
 # The function adds lines in multiple cells by first splitting data into several parts in which
-# each part corresponds to one factor (sector index) and then add lines in cells corresponding
-# to the part of data by calling `circos.lines`.
+# each part corresponds to one factor (sector index) and then add lines in cells by calling `circos.lines`.
 #
 # This function can be replaced by a ``for`` loop containing `circos.lines`.
 circos.trackLines = function(factors, x, y, track.index = get.cell.meta.data("track.index"),
     col = "black", lwd = par("lwd"), lty = par("lty"), type = "l", straight = FALSE,
-	area = FALSE, area.baseline = NULL, border = "black", baseline = NA,
+	area = FALSE, area.baseline = NULL, border = "black", baseline = "bottom",
     pt.col = par("col"), cex = par("cex"), pch = par("pch")) {
     
 	if(!is.null(area.baseline)) {
@@ -603,7 +630,7 @@ circos.trackLines = function(factors, x, y, track.index = get.cell.meta.data("tr
         circos.lines(nx, ny, sector.index = le[i],
                       track.index = track.index,
                       col = ncol, lwd = nlwd, lty = nlty, area = area[i], border = border[i],
-					  baseline = ifelse(is.na(baseline[i]), get.cell.meta.data("ylim", le[i], track.index)[1], baseline[i]),
+					  baseline = baseline[i],
                       pt.col = npt.col, cex = ncex, pch = npch, type = type, straight = straight)
             
     }
@@ -628,8 +655,8 @@ circos.trackLines = function(factors, x, y, track.index = get.cell.meta.data("tr
 #
 # == details
 # Currently, ``xleft``, ``ybottom``, ``xright``, ``ytop`` are all single values, which means
-# you can only draw one rectangle at once. Well, the name for this function is `circos.rect`
-# because if you imagin the plotting region as Cartesian coordinate, then it is rectangle.
+# you can only draw one rectangle at once. The name for this function is `circos.rect`
+# because if you imagine the plotting region as Cartesian coordinate, then it is rectangle.
 # in the polar coordinate, the up and bottom edge become two arcs.
 #
 # You just need to specify the coordinates of two diagonal points just similar as 
@@ -719,7 +746,8 @@ circos.polygon = function(x, y, sector.index = get.cell.meta.data("sector.index"
 # -sector.index Index for the sector
 # -track.index  Index for the track
 # -direction    deprecated, use ``facing`` instead.
-# -facing       Facing of text
+# -facing       Facing of text. Please refer to vignette for different settings 
+# -niceFacing   Should the facing of text be adjusted to fit human eyes?
 # -adj          Adjustment for text
 # -cex          Font size
 # -col          Font color
@@ -727,15 +755,15 @@ circos.polygon = function(x, y, sector.index = get.cell.meta.data("sector.index"
 # -...          Pass to `graphics::text`
 #
 # == details
-# The function is similar to `graphics::text`. All you need to note is the ``direction`` settings.
+# The function is similar to `graphics::text`. All you need to note is the ``facing`` settings.
 circos.text = function(x, y, labels, sector.index = get.cell.meta.data("sector.index"),
     track.index = get.cell.meta.data("track.index"), direction = NULL,
     facing = c("inside", "outside", "reverse.clockwise", "clockwise",
-	"downward", "bending"), adj = par("adj"), cex = 1, col = "black",
+	"downward", "bending"), niceFacing = FALSE, adj = par("adj"), cex = 1, col = "black",
 	font = par("font"), ...) {
     
 	if(length(x) != length(y)) {
-		stop("length of x and y differ.\n")
+		stop("Length of x and y differ.\n")
 	}
 	
     if(!has.cell(sector.index, track.index)) {
@@ -776,6 +804,53 @@ circos.text = function(x, y, labels, sector.index = get.cell.meta.data("sector.i
     }
     facing = match.arg(facing)[1]
     
+	if(niceFacing && facing %in% c("clockwise", "reverse.clockwise", "inside", "outside")) {
+		if(facing == "clockwise" || facing == "reverse.clockwise") {
+			degree = circlize(x, y, sector.index = sector.index, track.index = track.index)[, 1]
+			degree = degree %% 360
+			l1 = degree >= 90 & degree < 270  # should be reverse.clockwise
+			l2 = !l1  # should be clockwise
+			if(facing == "reverse.clockwise") {
+				adj1 = adj
+				adj2 = 1 - adj
+				facing1 = "reverse.clockwise"
+				facing2 = "clockwise"
+			} else {
+				adj1 = 1- adj
+				adj2 = adj
+				facing1 = "reverse.clockwise"
+				facing2 = "clockwise"
+			}
+		} else if(facing == "inside" || facing == "outside") {
+			degree = circlize(x, y, sector.index = sector.index, track.index = track.index)[, 1]
+			degree = degree %% 360
+			l1 = degree > 0 & degree < 180  # should be inside
+			l2 = !l1   # should be outside
+			if(facing == "inside") {
+				adj1 = adj
+				adj2 = 1 - adj
+				facing1 = "inside"
+				facing2 = "outside"
+			} else {
+				adj1 = 1 - adj
+				adj2 = adj
+				facing1 = "inside"
+				facing2 = "outside"
+			}
+		}
+		if(sum(l1)) {
+			circos.text(x[l1], y[l1], labels[l1], sector.index = sector.index,
+				track.index = track.index, facing = facing1, niceFacing = FALSE, adj = adj1,
+				cex = cex, col = col, ...)
+		}
+		
+		if(sum(l2)) {
+			circos.text(x[l2], y[l2], labels[l2], sector.index = sector.index,
+				track.index = track.index, facing = facing2, niceFacing = FALSE, adj = adj2,
+				cex = cex, col = col, ...)
+		}
+		return(invisible(NULL))
+	}
 	
 	if(facing == "bending") {
 		
@@ -838,6 +913,7 @@ circos.text = function(x, y, labels, sector.index = get.cell.meta.data("sector.i
 # -track.index  Index for the track
 # -direction    deprecated, use ``facing`` instead.
 # -facing       Facing of text
+# -niceFacing   Should the facing of text be adjusted to fit human eyes?
 # -adj          Adjustment for text
 # -cex          Font size
 # -col          Font color
@@ -845,13 +921,13 @@ circos.text = function(x, y, labels, sector.index = get.cell.meta.data("sector.i
 #
 # == details
 # The function adds texts in multiple cells by first splitting data into several parts in which
-# each part corresponds to one factor (sector index) and then add texts in cells corresponding
-# to the part of data by calling `circos.text`.
+# each part corresponds to one factor (sector index) and then add texts in cells by calling `circos.text`.
 #
 # This function can be replaced by a ``for`` loop containing `circos.text`.
 circos.trackText = function(factors, x, y, labels, track.index = get.cell.meta.data("track.index"),
     direction = NULL, facing = c("inside", "outside", "reverse.clockwise", "clockwise",
-	"downward", "bending"), adj = par("adj"), cex = 1, col = "black", font = par("font")) {
+	"downward", "bending"), niceFacing = FALSE, adj = par("adj"), cex = 1, col = "black", 
+	font = par("font")) {
     
     # basic check here
     if(length(x) != length(factors) || length(y) != length(factors)) {
@@ -886,8 +962,8 @@ circos.trackText = function(factors, x, y, labels, track.index = get.cell.meta.d
         nfont = font[l]
         circos.text(nx, ny, sector.index = le[i],
                       track.index = track.index, labels = nlabels,
-                      direction = direction, facing = facing, adj = adj,
-                      cex = ncex, col = ncol, font = nfont)
+                      direction = direction, facing = facing, niceFacing = niceFacing,
+					  adj = adj, cex = ncex, col = ncol, font = nfont)
             
     }
     return(invisible(NULL))
@@ -897,19 +973,20 @@ circos.trackText = function(factors, x, y, labels, track.index = get.cell.meta.d
 # Draw x-axis
 #
 # == param
-# -h                position of the x-axis, can be "top", "bottom" or a numeric value
-# -major.at         If it is numeric vector, it identifies the poisitions
+# -h                Position of the x-axis, can be "top", "bottom" or a numeric value
+# -major.at         If it is numeric vector, it identifies the positions
 #                   of the major ticks. It can exceed ``xlim`` value and the exceeding part
-#                   would be trimmed automatically. If it is ``NULL``, it would be calculated by `base::pretty` (about every 10 degrees there is a major tick).
+#                   would be trimmed automatically. If it is ``NULL``, about every 10 degrees there is a major tick.
 # -labels           labels of the major ticks. Also, the exceeding part would be trimmed automatically.
 # -major.tick       Whether to draw major tick. If it is set to ``FALSE``, there would be
-#                   no minor ticks either. 
+#                   no minor ticks.
 # -sector.index     Index for the sector
 # -track.index      Index for the track
 # -labels.font      font style for the axis labels
 # -labels.cex       font size for the axis labels
 # -labels.direction deprecated, use ``facing`` instead.
-# -labels.facing    facing of labels on axis
+# -labels.facing    facing of labels on axis, passing to `circos.text`
+# -labels.niceFacing Should facing of axis labels be human-easy
 # -direction        whether the axis ticks point to the outside or inside of the circle.
 # -minor.ticks      Number of minor ticks between two close major ticks.
 # -major.tick.percentage Length of the major ticks. It is the percentage to the height of the cell.
@@ -917,12 +994,15 @@ circos.trackText = function(factors, x, y, labels, track.index = get.cell.meta.d
 # -lwd              line width for ticks
 #
 # == details
-# It can only draw axis on x-direction.
+# It can only draw axes on x-direction.
+#
+# Currently, this package doesn't provide a function to add axes on y-direction. But it is easy
+# to implement by users with `circos.lines` and `circos.text`.
 circos.axis = function(h = "top", major.at = NULL, labels = TRUE, major.tick = TRUE,
 	sector.index = get.cell.meta.data("sector.index"),
 	track.index = get.cell.meta.data("track.index"),
 	labels.font = par("font"), labels.cex = par("cex"),
-	labels.facing = "inside", labels.direction = NULL,
+	labels.facing = "inside", labels.direction = NULL, labels.niceFacing = TRUE,
 	direction = c("outside", "inside"), minor.ticks = 4,
 	major.tick.percentage = 0.1, labels.away.percentage = 0.05, lwd = par("lwd")) {
 	
@@ -953,10 +1033,9 @@ circos.axis = function(h = "top", major.at = NULL, labels = TRUE, major.tick = T
 	}
 	
 	if(is.null(major.at)) {
-		# every 10 degrees there is a major tick. This is hard coded.
-		# start.degree - end.degree is always a positive value.
-		n = floor(abs(sector.data["start.degree"] - sector.data["end.degree"]) / 10)
-		major.at = pretty(xlim, n = n)
+		major.by = .default.major.by(sector.index, track.index)
+		major.at = seq(floor(xlim[1]/major.by)*major.by, xlim[2], by = major.by)
+		major.at = c(major.at, major.at[length(major.at)] + major.by)
 	}
 	
 	minor.at = NULL
@@ -1025,14 +1104,14 @@ circos.axis = function(h = "top", major.at = NULL, labels = TRUE, major.tick = T
 			circos.text(major.at[i], h + (major.tick.length+yrange*labels.away.percentage)*ifelse(direction == "outside", 1, -1),
 			           labels = major.at[i], adj = labels.adj,
 			           font = labels.font, cex = labels.cex, sector.index = sector.index, track.index = track.index,
-			           facing = labels.facing)
+			           facing = labels.facing, niceFacing = labels.niceFacing)
 		} else if(is.logical(labels) && !labels) {
                           
         } else if(length(labels)) {
 			circos.text(major.at[i], h + (major.tick.length+yrange*labels.away.percentage)*ifelse(direction == "outside", 1, -1),
 			            labels = labels[i], adj = labels.adj,
 			            font = labels.font, cex = labels.cex, sector.index = sector.index, track.index = track.index,
-				        facing = labels.facing)
+				        facing = labels.facing, niceFacing = labels.niceFacing)
 		}				
 		
 	}
@@ -1051,6 +1130,18 @@ circos.axis = function(h = "top", major.at = NULL, labels = TRUE, major.tick = T
 	return(invisible(NULL))
 }
 
+.default.major.by = function(sector.index = get.cell.meta.data("sector.index"),
+	track.index = get.cell.meta.data("track.index")) {
+	# start.degree - end.degree is always a positive value.
+	d = circos.par("major.by.degree")
+	cell.start.degre = get.cell.meta.data("cell.start.degree", sector.index, track.index)
+	tm = reverse.circlize(c(cell.start.degre, cell.start.degre-d), rep(get.cell.meta.data("cell.bottom.radius", sector.index, track.index), 2))
+	major.by = abs(tm[1, 1] - tm[2, 1])
+	digits = as.numeric(gsub("^.*e([+-]\\d+)$", "\\1", sprintf("%e", major.by)))
+	major.by = round(major.by, digits = -1*digits)
+	return(major.by)
+}	
+		
 #####################################################################
 #
 # simulate high-level graphic functions such as barplot, hist, boxplot ...
@@ -1063,9 +1154,9 @@ circos.axis = function(h = "top", major.at = NULL, labels = TRUE, major.tick = T
 # == param
 # -factors      Factors which represent the categories of data
 # -x            Data on the x-axis
-# -track.index  Index for the track which is goning to be updated. Setting it to ``NULL`` means
+# -track.index  Index for the track which is going to be updated. Setting it to ``NULL`` means
 #               creating the plotting regions in the next newest track.
-# -track.height Height of the track. It is the percentage to the radius of the unit circls.
+# -track.height Height of the track. It is the percentage to the radius of the unit circle.
 #               If to update a track, this argument is disabled.
 # -force.ylim   Whether to force all cells in the track to share the same ``ylim``. Btw, ``ylim`` is calculated automatically.
 # -col          Filled color for histogram
@@ -1073,7 +1164,7 @@ circos.axis = function(h = "top", major.at = NULL, labels = TRUE, major.tick = T
 # -lty          Line style for histogram
 # -lwd          Line width for histogram
 # -bg.col       Background color for the plotting regions
-# -bg.border    Color for the boder of the plotting regions
+# -bg.border    Color for the border of the plotting regions
 # -bg.lty       Line style for the border of the plotting regions
 # -bg.lwd       Line width for the border of the plotting regions
 # -breaks       see `graphics::hist`
@@ -1082,8 +1173,8 @@ circos.axis = function(h = "top", major.at = NULL, labels = TRUE, major.tick = T
 # -draw.density   whether draw density lines instead of histogram bars.
 # 
 # == details
-# It draw histogram in cells among a whole track. It is also an example to show how to draw self-defined
-# figures by this package.
+# It draw histogram in cells among a whole track. It is also an example to show how to add self-defined
+# high-level graphics by this package.
 circos.trackHist = function(factors, x, track.height = circos.par("default.track.height"),
     track.index = NULL, force.ylim = TRUE, col = ifelse(draw.density, "black", NA),
 	border = "black", lty = par("lty"), lwd = par("lwd"),
