@@ -1,4 +1,6 @@
 
+op = par(no.readonly = TRUE)
+
 par(mar = c(1.5, 1.5, 1.5, 1.5))
 # highlight 
 circos.par("default.track.height" = 0.1, cell.padding = c(0, 0, 0, 0))
@@ -30,3 +32,5 @@ highlight.chromosome("chr3", col = NA, border = "red", lwd = 2, padding = c(0.05
 highlight.chromosome("chr5", col = "#0000FF40", track.index = c(2, 4, 5))
 highlight.chromosome("chr7", col = NA, border = "green", lwd = 2, track.index = c(2, 4, 5))
 circos.clear()
+
+par(op)
