@@ -1,3 +1,6 @@
+
+op = par(no.readonly = TRUE)
+
 library(circlize)
 par(mar = c(1, 1, 1, 1))
 factors = letters[1:4]
@@ -12,3 +15,5 @@ for(i in 1:4) {
     par(new = TRUE)
 }
 par(new = FALSE)
+
+par(op)
